@@ -5,31 +5,34 @@
  */
 
 #include "fd.h"
-
-#ifndef	_NOPTY
-
-#include <signal.h>
 #include "termio.h"
+#include "kconv.h"
 #include "func.h"
-#include "termemu.h"
 
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 #include "system.h"
 #endif
+#ifdef	DEP_PTY
+#include "termemu.h"
+#endif
 
-#define	MAXESCPARAM	16
-#define	MAXESCCHAR	4
-#define	MAXTABSTOP	255
-#define	MAXLASTUCS2	(MAXNFLEN - 1)
+#define	MAXESCPARAM		16
+#define	MAXESCCHAR		4
+#define	MAXTABSTOP		255
+#define	MAXLASTUCS2		(MAXNFLEN - 1)
+
+typedef struct _ptystat_t {
+	short x, y;
+	short fg, bg;
+	u_short attr;
+} ptystat_t;
 
 typedef struct _ptyterm_t {
-	short cur_x, cur_y;
+	ptystat_t cur, save;
 	short min_x, min_y;
 	short max_x, max_y;
-	short save_x, save_y;
 	short min_scroll, max_scroll;
 	short last1, last2;
-	short fg, bg;
 	u_char escmode;
 	u_char nparam;
 	u_char nchar;
@@ -37,49 +40,34 @@ typedef struct _ptyterm_t {
 	short escparam[MAXESCPARAM];
 	u_char escchar[MAXESCCHAR];
 	u_short tabstop[MAXTABSTOP];
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	u_short last_ucs2[MAXLASTUCS2];
 #endif
-	u_short attr;
 	u_short termflags;
 	char codeselect[MAXESCCHAR + 2];
 } ptyterm_t;
 
-#define	A_BOLD		00001
-#define	A_REVERSE	00002
-#define	A_DIM		00004
-#define	A_BLINK		00010
-#define	A_STANDOUT	00020
-#define	A_UNDERLINE	00040
-#define	A_INVISIBLE	00100
-#define	T_NOAUTOMARGIN	00001
-#define	T_MULTIBYTE	00002
-#define	T_NOAPPLIKEY	00004
-#define	T_NOAPPLICURSOR	00010
-#define	T_CODEALT	00020
-#define	T_LOCKED	00040
+#define	A_BOLD			00001
+#define	A_REVERSE		00002
+#define	A_DIM			00004
+#define	A_BLINK			00010
+#define	A_STANDOUT		00020
+#define	A_UNDERLINE		00040
+#define	A_INVISIBLE		00100
+#define	T_NOAUTOMARGIN		00001
+#define	T_MULTIBYTE		00002
+#define	T_NOAPPLIKEY		00004
+#define	T_NOAPPLICURSOR		00010
+#define	T_CODEALT		00020
+#define	T_LOCKED		00040
+
+#ifdef	DEP_PTY
 
 extern int hideclock;
 extern int fdmode;
 extern int wheader;
 extern int emufd;
 
-static ptyterm_t pty[MAXWINDOWS + 1];
-static short last_x = (short)-1;
-static short last_y = (short)-1;
-static short last_fg = (short)-1;
-static short last_bg = (short)-1;
-static u_short last_attr = (u_short)0;
-static u_short last_termflags = (u_short)0;
-static char last_codeselect[MAXESCCHAR + 3] = "\033(B";
-#ifdef	SIGWINCH
-static int ptywinched = 0;
-#endif
-#ifndef	_NOKANJICONV
-static u_char ptyungetbuf[MAXUTF8LEN * MAXNFLEN * sizeof(short)];
-static int ptyungetnum = 0;
-#endif
-
 static VOID NEAR resettermattr __P_((int));
 static VOID NEAR resettermcode __P_((int));
 static VOID NEAR resettabstop __P_((int));
@@ -93,11 +81,7 @@ static int wintr __P_((VOID_A));
 static VOID NEAR evalsignal __P_((VOID_A));
 #endif
 static int NEAR convkey __P_((int, keyseq_t *));
-#ifdef	_NOKANJICONV
-#define	ptyrecvbuf(b, n)	recvbuf(emufd, b, n)
-#define	ptyrecvword(n)		recvword(emufd, n)
-#define	ptyrecvstring(c)	recvstring(emufd, c)
-#else	/* !_NOKANJICONV */
+#ifdef	DEP_KCONV
 static int NEAR ptyrecvbuf __P_((VOID_P, int));
 static int NEAR ptyrecvword __P_((int *));
 static int NEAR ptyrecvstring __P_((char **));
@@ -107,7 +91,11 @@ static u_int NEAR ptygetucs2 __P_((int));
 static VOID NEAR getiocode __P_((int, int *, int *));
 static CONST char *NEAR ptykconv __P_((char *, char *, int, int));
 static u_int NEAR evalnf __P_((int, char *));
-#endif	/* !_NOKANJICONV */
+#else	/* !DEP_KCONV */
+#define	ptyrecvbuf(b, n)	recvbuf(emufd, b, n)
+#define	ptyrecvword(n)		recvword(emufd, n)
+#define	ptyrecvstring(c)	recvstring(emufd, c)
+#endif	/* !DEP_KCONV */
 static VOID NEAR evalscroll __P_((int, int, int));
 static VOID NEAR evallf __P_((int));
 static VOID NEAR outputnormal __P_((int, char *, int));
@@ -121,13 +109,29 @@ static int NEAR chgattr __P_((int, int));
 static int NEAR directoutput __P_((int));
 static int NEAR evalinput __P_((VOID_A));
 
+static ptyterm_t pty[MAXWINDOWS + 1];
+static short last_x = (short)-1;
+static short last_y = (short)-1;
+static short last_fg = (short)-1;
+static short last_bg = (short)-1;
+static u_short last_attr = (u_short)0;
+static u_short last_termflags = (u_short)0;
+static char last_codeselect[MAXESCCHAR + 3] = "\033(B";
+#ifdef	SIGWINCH
+static int ptywinched = 0;
+#endif
+#ifdef	DEP_KCONV
+static u_char ptyungetbuf[MAXUTF8LEN * MAXNFLEN * sizeof(short)];
+static int ptyungetnum = 0;
+#endif
+
 
 static VOID NEAR resettermattr(w)
 int w;
 {
 	if (w >= 0) {
-		pty[w].fg = pty[w].bg = (short)-1;
-		pty[w].attr = (u_short)0;
+		pty[w].cur.attr = (u_short)0;
+		pty[w].cur.fg = pty[w].cur.bg = (short)-1;
 	}
 	else if (last_attr || last_fg >= (short)0 || last_bg >= (short)0) {
 		putterm(T_NORMAL);
@@ -140,7 +144,7 @@ static VOID NEAR resettermcode(w)
 int w;
 {
 	pty[w].termflags &= ~T_CODEALT;
-	strcpy(pty[w].codeselect, "(B");
+	Xstrcpy(pty[w].codeselect, "(B");
 }
 
 static VOID NEAR resettabstop(w)
@@ -158,7 +162,7 @@ int w;
 VOID resetptyterm(w, clean)
 int w, clean;
 {
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	int i;
 #endif
 	int min_x, min_y, max_y;
@@ -172,10 +176,10 @@ int w, clean;
 
 	pty[w].min_x = (short)0;
 	pty[w].max_x = n_column;
-#ifdef	_NOORIGSHELL
-	if (w == MAXWINDOWS)
-#else
+#ifdef	DEP_ORIGSHELL
 	if (w == MAXWINDOWS || isshptymode())
+#else
+	if (w == MAXWINDOWS)
 #endif
 	{
 		pty[w].min_y = (short)0;
@@ -187,13 +191,13 @@ int w, clean;
 	}
 
 	if (clean) {
-		pty[w].cur_x = pty[w].save_x = pty[w].min_x;
-		pty[w].cur_y = pty[w].save_y = pty[w].max_y - 1;
+		pty[w].cur.x = pty[w].min_x;
+		pty[w].cur.y = pty[w].max_y - 1;
 		pty[w].min_scroll = pty[w].min_y;
 		pty[w].max_scroll = pty[w].max_y - 1;
 		pty[w].escmode = '\0';
 		pty[w].last1 = pty[w].last2 = (short)-1;
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 		for (i = 0; i < MAXLASTUCS2; i++)
 			pty[w].last_ucs2[i] = (u_short)0;
 #endif
@@ -201,16 +205,18 @@ int w, clean;
 		resettermattr(w);
 		resettermcode(w);
 		resettabstop(w);
+		memcpy((char *)&(pty[w].save), (char *)&(pty[w].cur),
+			sizeof(pty[w].save));
 	}
 	else {
-		pty[w].cur_x += pty[w].min_x - min_x;
-		pty[w].cur_y += pty[w].min_y - min_y;
-		pty[w].save_x += pty[w].save_x - min_x;
-		pty[w].save_y += pty[w].save_y - min_y;
+		pty[w].cur.x += pty[w].min_x - min_x;
+		pty[w].cur.y += pty[w].min_y - min_y;
+		pty[w].save.x += pty[w].min_x - min_x;
+		pty[w].save.y += pty[w].min_y - min_y;
 		pty[w].min_scroll += pty[w].min_y - min_y;
 		pty[w].max_scroll += pty[w].max_y - max_y;
-		biasxy(w, &(pty[w].cur_x), &(pty[w].cur_y));
-		biasxy(w, &(pty[w].save_x), &(pty[w].save_y));
+		biasxy(w, &(pty[w].cur.x), &(pty[w].cur.y));
+		biasxy(w, &(pty[w].save.x), &(pty[w].save.y));
 		biasxy(w, NULL, &(pty[w].min_scroll));
 		biasxy(w, NULL, &(pty[w].max_scroll));
 	}
@@ -233,39 +239,31 @@ short *xp, *yp;
 static VOID NEAR settermattr(w)
 int w;
 {
-	char *cp;
-
-	if ((last_fg >= (short)0 && pty[w].fg < (short)0)
-	|| (last_bg >= (short)0 && pty[w].bg < (short)0))
+	if ((last_fg >= (short)0 && pty[w].cur.fg < (short)0)
+	|| (last_bg >= (short)0 && pty[w].cur.bg < (short)0))
 		resettermattr(-1);
 
-	if (last_attr != pty[w].attr) {
+	if (last_attr != pty[w].cur.attr) {
 		putterm(T_NORMAL);
 		last_fg = last_bg = (short)-1;
-		if (pty[w].attr & A_BOLD) putterm(T_BOLD);
-		if (pty[w].attr & A_REVERSE) putterm(T_REVERSE);
-		if (pty[w].attr & A_DIM) putterm(T_DIM);
-		if (pty[w].attr & A_BLINK) putterm(T_BLINK);
-		if (pty[w].attr & A_STANDOUT) putterm(T_STANDOUT);
-		if (pty[w].attr & A_UNDERLINE) putterm(T_UNDERLINE);
-		last_attr = pty[w].attr;
+		if (pty[w].cur.attr & A_BOLD) putterm(T_BOLD);
+		if (pty[w].cur.attr & A_REVERSE) putterm(T_REVERSE);
+		if (pty[w].cur.attr & A_DIM) putterm(T_DIM);
+		if (pty[w].cur.attr & A_BLINK) putterm(T_BLINK);
+		if (pty[w].cur.attr & A_STANDOUT) putterm(T_STANDOUT);
+		if (pty[w].cur.attr & A_UNDERLINE) putterm(T_UNDERLINE);
+		last_attr = pty[w].cur.attr;
 	}
 
-	if (last_fg != pty[w].fg && pty[w].fg >= (short)0) {
-		if ((cp = tparamstr(termstr[T_FGCOLOR], pty[w].fg, 0))) {
-			tputs2(cp, 1);
-			free(cp);
-		}
-		else cprintf2("\033[%dm", pty[w].fg + ANSI_NORMAL);
-		last_fg = pty[w].fg;
+	if (last_fg != pty[w].cur.fg && pty[w].cur.fg >= (short)0) {
+		if (tputparam(T_FGCOLOR, pty[w].cur.fg, 0, 1) < 0)
+			tprintf("\033[%dm", 1, pty[w].cur.fg + ANSI_NORMAL);
+		last_fg = pty[w].cur.fg;
 	}
-	if (last_bg != pty[w].bg && pty[w].bg >= (short)0) {
-		if ((cp = tparamstr(termstr[T_BGCOLOR], pty[w].bg, 0))) {
-			tputs2(cp, 1);
-			free(cp);
-		}
-		else cprintf2("\033[%dm", pty[w].bg + ANSI_REVERSE);
-		last_bg = pty[w].bg;
+	if (last_bg != pty[w].cur.bg && pty[w].cur.bg >= (short)0) {
+		if (tputparam(T_BGCOLOR, pty[w].cur.bg, 0, 1) < 0)
+			tprintf("\033[%dm", 1, pty[w].cur.bg + ANSI_REVERSE);
+		last_bg = pty[w].cur.bg;
 	}
 }
 
@@ -274,17 +272,17 @@ int w;
 {
 	if ((last_termflags & T_CODEALT) != (pty[w].termflags & T_CODEALT)) {
 		if (pty[w].termflags & T_CODEALT) {
-			putch2('\016');
+			VOID_C Xputch('\016');
 			last_termflags |= T_CODEALT;
 		}
 		else {
-			putch2('\017');
+			VOID_C Xputch('\017');
 			last_termflags &= ~T_CODEALT;
 		}
 	}
 	if (strcmp(&(last_codeselect[1]), pty[w].codeselect)) {
 		last_codeselect[0] = '\033';
-		strcpy(&(last_codeselect[1]), pty[w].codeselect);
+		Xstrcpy(&(last_codeselect[1]), pty[w].codeselect);
 		tputs2(last_codeselect, 1);
 	}
 }
@@ -292,41 +290,44 @@ int w;
 static VOID NEAR surelocate(w, forced)
 int w, forced;
 {
-	if (!forced && pty[w].cur_x == last_x && pty[w].cur_y == last_y)
+	if (!forced && pty[w].cur.x == last_x && pty[w].cur.y == last_y)
 		return;
 
 	resettermattr(-1);
-	locate(pty[w].cur_x, pty[w].cur_y);
-	last_x = pty[w].cur_x;
-	last_y = pty[w].cur_y;
+	locate(pty[w].cur.x, pty[w].cur.y);
+	last_x = pty[w].cur.x;
+	last_y = pty[w].cur.y;
 }
 
 static VOID NEAR reallocate(w, x, y)
+int w, x, y;
 {
-	if (pty[w].attr || pty[w].fg >= (short)0 || pty[w].bg >= (short)0) {
+	if (pty[w].cur.attr
+	|| pty[w].cur.fg >= (short)0 || pty[w].cur.bg >= (short)0) {
 		surelocate(w, 1);
 		settermattr(w);
 	}
 
-	pty[w].cur_x = x;
-	pty[w].cur_y = y;
-	biasxy(w, &(pty[w].cur_x), &(pty[w].cur_y));
-	locate(pty[w].cur_x, pty[w].cur_y);
+	pty[w].cur.x = x;
+	pty[w].cur.y = y;
+	biasxy(w, &(pty[w].cur.x), &(pty[w].cur.y));
+	locate(pty[w].cur.x, pty[w].cur.y);
 	tflush();
-	last_x = pty[w].cur_x;
-	last_y = pty[w].cur_y;
+	last_x = pty[w].cur.x;
+	last_y = pty[w].cur.y;
 }
 
 #ifdef	SIGWINCH
 static int wintr(VOID_A)
 {
+	sigblk_t old;
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGWINCH, SIG_IGN);
+	old = suspendsignal(SIGWINCH, (sigcst_t)wintr);
 	pollscreen(1);
 	ptywinched++;
-	signal2(SIGWINCH, (sigcst_t)wintr);
+	resumesignal(SIGWINCH, old);
 	errno = duperrno;
 
 	return(0);
@@ -343,14 +344,14 @@ static VOID NEAR evalsignal(VOID_A)
 		xmax = pty[i].max_x - pty[i].min_x;
 		ymax = pty[i].max_y - pty[i].min_y;
 		resetptyterm(i, 0);
-		VOID_C setwsize(ptylist[i].fd,
+		setwsize(ptylist[i].fd,
 			pty[i].max_x - pty[i].min_x,
 			pty[i].max_y - pty[i].min_y);
 
 		if (xmax != pty[i].max_x - pty[i].min_x
 		|| ymax != pty[i].max_y - pty[i].min_y) {
-			pty[i].cur_x = pty[i].save_x = pty[i].min_x;
-			pty[i].cur_y = pty[i].save_y = pty[i].max_y - 1;
+			pty[i].cur.x = pty[i].save.x = pty[i].min_x;
+			pty[i].cur.y = pty[i].save.y = pty[i].max_y - 1;
 		}
 	}
 	resetptyterm(i, 0);
@@ -386,7 +387,7 @@ keyseq_t *kp;
 	}
 
 	if ((pty[w].termflags & T_NOAPPLIKEY)
-	&& (kp -> code >= K_F('*') && kp -> code <= K_DL)) {
+	&& (kp -> code >= K_F('*') && kp -> code < K_DL)) {
 		if (kp -> code == K_F('?')) kp -> code = K_CR;
 		else kp -> code -= K_F0;
 		kp -> len = (u_char)0;
@@ -396,7 +397,7 @@ keyseq_t *kp;
 	return((getdefkeyseq(kp) < 0) ? getkeyseq(kp) : 0);
 }
 
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 static int NEAR ptyrecvbuf(buf, nbytes)
 VOID_P buf;
 int nbytes;
@@ -439,9 +440,9 @@ char **cpp;
 	if (ptyrecvbuf(&cp, sizeof(cp)) < 0) return(-1);
 	if (cp) {
 		if (ptyrecvbuf(&len, sizeof(len)) < 0) return(-1);
-		cp = malloc2(len + 1);
+		cp = Xmalloc(len + 1);
 		if (ptyrecvbuf(cp, len) < 0) {
-			free(cp);
+			Xfree(cp);
 			return(-1);
 		}
 		cp[len] = '\0';
@@ -550,7 +551,7 @@ char *buf;
 		u = ucs2fromutf8((u_char *)buf, NULL);
 		if (incode != M_UTF8) return(u);
 		ubuf[i++] = u;
-		next = selectpty(1, &(ptylist[w].fd), NULL, WAITMETA * 1000L);
+		next = selectpty(1, &(ptylist[w].fd), NULL, WAITKANJI * 1000L);
 	}
 	ubuf[i] = (u_short)0;
 
@@ -573,26 +574,26 @@ char *buf;
 
 	return((u_int)0);
 }
-#endif	/* !_NOKANJICONV */
+#endif	/* DEP_KCONV */
 
 static VOID NEAR evalscroll(w, n, c)
 int w, n, c;
 {
 	resettermattr(-1);
-	regionscroll(n, c, pty[w].cur_x, pty[w].cur_y,
+	regionscroll(n, c, pty[w].cur.x, pty[w].cur.y,
 		pty[w].min_scroll, pty[w].max_scroll);
 	tflush();
-	last_x = pty[w].cur_x;
-	last_y = pty[w].cur_y;
+	last_x = pty[w].cur.x;
+	last_y = pty[w].cur.y;
 }
 
 static VOID NEAR evallf(w)
 int w;
 {
-	if (pty[w].cur_y < pty[w].max_y
-	&& pty[w].cur_y == pty[w].max_scroll)
+	if (pty[w].cur.y < pty[w].max_y
+	&& pty[w].cur.y == pty[w].max_scroll)
 		evalscroll(w, C_SCROLLFORW, 1);
-	else reallocate(w, pty[w].cur_x, pty[w].cur_y + 1);
+	else reallocate(w, pty[w].cur.x, pty[w].cur.y + 1);
 }
 
 static VOID NEAR outputnormal(w, buf, width)
@@ -600,14 +601,14 @@ int w;
 char *buf;
 int width;
 {
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	char buf2[MAXKANJIBUF + 1];
 	int incode, outcode;
 #endif
 	CONST char *cp;
 
-	if (pty[w].cur_x + width > pty[w].max_x) {
-		pty[w].cur_x = pty[w].min_x;
+	if (pty[w].cur.x + width > pty[w].max_x) {
+		pty[w].cur.x = pty[w].min_x;
 		evallf(w);
 	}
 
@@ -615,11 +616,11 @@ int width;
 	settermattr(w);
 	settermcode(w);
 	cp = buf;
-	if (pty[w].attr & A_INVISIBLE) {
+	if (pty[w].cur.attr & A_INVISIBLE) {
 		memset(buf, ' ', width);
 		buf[width] = '\0';
 	}
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	else if (!(pty[w].termflags & T_MULTIBYTE)) {
 		getiocode(w, &incode, &outcode);
 		if (incode != outcode)
@@ -627,13 +628,13 @@ int width;
 	}
 #endif
 
-	cputs2(cp);
+	Xcputs(cp);
 	tflush();
-	pty[w].cur_x += width;
+	pty[w].cur.x += width;
 	last_x += width;
 
-	if (n_lastcolumn < n_column && pty[w].cur_x >= pty[w].max_x) {
-		pty[w].cur_x = pty[w].min_x;
+	if (n_lastcolumn < n_column && pty[w].cur.x >= pty[w].max_x) {
+		pty[w].cur.x = pty[w].min_x;
 		evallf(w);
 	}
 }
@@ -641,19 +642,19 @@ int width;
 static VOID NEAR evalnormal(w, c)
 int w, c;
 {
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	u_int u;
 	int incode;
 #endif
 	char buf[MAXKANJIBUF + 1];
 	int i, width;
 
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	getiocode(w, &incode, NULL);
 #endif
 
 	if ((pty[w].termflags & T_NOAUTOMARGIN)
-	&& pty[w].cur_x >= pty[w].max_x - 1)
+	&& pty[w].cur.x >= pty[w].max_x - 1)
 		return;
 
 	i = 0;
@@ -661,11 +662,7 @@ int w, c;
 	if (pty[w].last1 >= (short)0) {
 		buf[i++] = pty[w].last1;
 		if (pty[w].termflags & T_MULTIBYTE) width++;
-#ifdef	_NOKANJICONV
-# ifdef	CODEEUC
-		else if (pty[w].last1 == C_EKANA) /*EMPTY*/;
-# endif
-#else	/* !_NOKANJICONV */
+#ifdef	DEP_KCONV
 		else if (incode == EUC) {
 			if (pty[w].last1 != C_EKANA) width++;
 		}
@@ -678,21 +675,25 @@ int w, c;
 				return;
 			}
 		}
-#endif	/* !_NOKANJICONV */
+#else	/* !DEP_KCONV */
+# ifdef	CODEEUC
+		else if (pty[w].last1 == C_EKANA) /*EMPTY*/;
+# endif
+#endif	/* !DEP_KCONV */
 		else width++;
 	}
-#ifdef	_NOKANJICONV
+#ifdef	DEP_KCONV
+	else if ((pty[w].termflags & T_MULTIBYTE)
+	|| (incode == EUC && (c == C_EKANA || iseuc(c)))
+	|| (incode == SJIS && issjis1(c))
+	|| (incode >= UTF8 && ismsb(c)))
+#else	/* !DEP_KCONV */
 # ifdef	CODEEUC
 	else if (c == C_EKANA || iseuc(c))
 # else
 	else if (issjis1(c))
 # endif
-#else	/* !_NOKANJICONV */
-	else if ((pty[w].termflags & T_MULTIBYTE)
-	|| (incode == EUC && (c == C_EKANA || iseuc(c)))
-	|| (incode == SJIS && issjis1(c))
-	|| (incode >= UTF8 && ismsb(c)))
-#endif	/* !_NOKANJICONV */
+#endif	/* !DEP_KCONV */
 	{
 		pty[w].last1 = c;
 		pty[w].last2 = (short)-1;
@@ -702,7 +703,7 @@ int w, c;
 	buf[i++] = c;
 	buf[i] = '\0';
 
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	if (incode >= UTF8) {
 		if (!(u = evalnf(w, buf))) return;
 		if (iswucs2(u)) width++;
@@ -711,7 +712,7 @@ int w, c;
 
 	outputnormal(w, buf, width);
 	pty[w].last1 = pty[w].last2 = (short)-1;
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	for (i = 0; i < MAXLASTUCS2; i++) pty[w].last_ucs2[i] = (u_short)0;
 #endif
 }
@@ -723,17 +724,17 @@ int w, c;
 
 	switch (c) {
 		case '\b':
-			reallocate(w, pty[w].cur_x - 1, pty[w].cur_y);
+			reallocate(w, pty[w].cur.x - 1, pty[w].cur.y);
 			break;
 		case '\t':
 			for (i = 0; i < pty[w].ntabstop; i++)
-				if (pty[w].cur_x < pty[w].tabstop[i]) break;
+				if (pty[w].cur.x < pty[w].tabstop[i]) break;
 			i = (i < pty[w].ntabstop)
 				? pty[w].tabstop[i] : pty[w].max_x - 1;
-			reallocate(w, i, pty[w].cur_y);
+			reallocate(w, i, pty[w].cur.y);
 			break;
 		case '\r':
-			reallocate(w, pty[w].min_x, pty[w].cur_y);
+			reallocate(w, pty[w].min_x, pty[w].cur.y);
 			break;
 		case '\n':
 		case '\v':
@@ -766,13 +767,13 @@ int w, c;
 	pty[w].escmode = '\0';
 	switch (c) {
 		case '7':
-			pty[w].save_x = pty[w].cur_x;
-			pty[w].save_y = pty[w].cur_y;
+			memcpy((char *)&(pty[w].save), (char *)&(pty[w].cur),
+				sizeof(pty[w].save));
 			break;
 		case '8':
-			pty[w].cur_x = pty[w].save_x;
-			pty[w].cur_y = pty[w].save_y;
-			biasxy(w, &(pty[w].cur_x), &(pty[w].cur_y));
+			memcpy((char *)&(pty[w].cur), (char *)&(pty[w].save),
+				sizeof(pty[w].cur));
+			biasxy(w, &(pty[w].cur.x), &(pty[w].cur.y));
 			surelocate(w, 1);
 			tflush();
 			break;
@@ -786,36 +787,38 @@ int w, c;
 			evallf(w);
 			break;
 		case 'E':
-			pty[w].cur_x = pty[w].min_x;
+			pty[w].cur.x = pty[w].min_x;
 			evallf(w);
 			break;
 		case 'H':
 			if (pty[w].ntabstop >= (u_char)MAXTABSTOP) break;
 			for (i = 0; i < pty[w].ntabstop; i++)
-				if (pty[w].cur_x <= pty[w].tabstop[i]) break;
+				if (pty[w].cur.x <= pty[w].tabstop[i]) break;
 			if (i >= pty[w].ntabstop)
 				pty[w].tabstop[(pty[w].ntabstop)++] =
-					pty[w].cur_x;
-			else if (pty[w].cur_x == pty[w].tabstop[i]) break;
+					pty[w].cur.x;
+			else if (pty[w].cur.x == pty[w].tabstop[i]) break;
 			else {
 				memmove((char *)&(pty[w].tabstop[i + 1]),
 					(char *)&(pty[w].tabstop[i]),
 					((pty[w].ntabstop)++ - i)
 						* sizeof(u_short));
-				pty[w].tabstop[i] = pty[w].cur_x;
+				pty[w].tabstop[i] = pty[w].cur.x;
 			}
 			break;
 		case 'M':
-			if (pty[w].cur_y >= pty[w].min_y
-			&& pty[w].cur_y == pty[w].min_scroll)
+			if (pty[w].cur.y >= pty[w].min_y
+			&& pty[w].cur.y == pty[w].min_scroll)
 				evalscroll(w, L_INSERT, 1);
-			else reallocate(w, pty[w].cur_x, pty[w].cur_y - 1);
+			else reallocate(w, pty[w].cur.x, pty[w].cur.y - 1);
 			break;
 		case 'c':
 			pty[w].termflags = (u_short)0;
 			resettermattr(w);
 			resettermcode(w);
 			resettabstop(w);
+			memcpy((char *)&(pty[w].save), (char *)&(pty[w].cur),
+				sizeof(pty[w].save));
 			surelocate(w, 1);
 			tflush();
 			pty[w].min_scroll = pty[w].min_y;
@@ -872,25 +875,25 @@ int w, c, fd;
 			if (pty[w].escmode != ']') break;
 			n = pty[w].escparam[0];
 			if (n <= 0) n = 1;
-			reallocate(w, pty[w].cur_x, pty[w].cur_y - n);
+			reallocate(w, pty[w].cur.x, pty[w].cur.y - n);
 			break;
 		case 'B':
 			if (pty[w].escmode != ']') break;
 			n = pty[w].escparam[0];
 			if (n <= 0) n = 1;
-			reallocate(w, pty[w].cur_x, pty[w].cur_y + n);
+			reallocate(w, pty[w].cur.x, pty[w].cur.y + n);
 			break;
 		case 'C':
 			if (pty[w].escmode != ']') break;
 			n = pty[w].escparam[0];
 			if (n <= 0) n = 1;
-			reallocate(w, pty[w].cur_x + n, pty[w].cur_y);
+			reallocate(w, pty[w].cur.x + n, pty[w].cur.y);
 			break;
 		case 'D':
 			if (pty[w].escmode != ']') break;
 			n = pty[w].escparam[0];
 			if (n <= 0) n = 1;
-			reallocate(w, pty[w].cur_x - n, pty[w].cur_y);
+			reallocate(w, pty[w].cur.x - n, pty[w].cur.y);
 			break;
 		case 'H':
 		case 'f':
@@ -899,8 +902,6 @@ int w, c, fd;
 				? 0 : pty[w].escparam[1] - 1);
 			y = pty[w].min_y + (pty[w].escparam[0] - 1);
 			reallocate(w, x, y);
-			pty[w].save_x = pty[w].cur_x;
-			pty[w].save_y = pty[w].cur_y;
 			break;
 		case 'm':
 			if (pty[w].escmode != ']') break;
@@ -908,65 +909,67 @@ int w, c, fd;
 			switch (pty[w].escparam[i]) {
 				case -1:
 				case 0:
-					pty[w].attr = (u_short)0;
-					pty[w].fg = pty[w].bg = (short)-1;
+					pty[w].cur.attr = (u_short)0;
+					pty[w].cur.fg =
+					pty[w].cur.bg = (short)-1;
 					break;
 				case 1:
-					pty[w].attr |= A_BOLD;
+					pty[w].cur.attr |= A_BOLD;
 					break;
 				case 2:
-					pty[w].attr |= A_DIM;
+					pty[w].cur.attr |= A_DIM;
 					break;
 				case 4:
-					pty[w].attr |= A_UNDERLINE;
+					pty[w].cur.attr |= A_UNDERLINE;
 					break;
 				case 5:
-					pty[w].attr |= A_BLINK;
+					pty[w].cur.attr |= A_BLINK;
 					break;
 				case 7:
-					pty[w].attr |= A_REVERSE;
+					pty[w].cur.attr |= A_REVERSE;
 					break;
 				case 8:
-					pty[w].attr |= A_INVISIBLE;
+					pty[w].cur.attr |= A_INVISIBLE;
 					break;
 				case 22:
-					pty[w].attr &= ~A_BOLD;
+					pty[w].cur.attr &= ~A_BOLD;
 					break;
 				case 24:
-					pty[w].attr &= ~A_UNDERLINE;
+					pty[w].cur.attr &= ~A_UNDERLINE;
 					break;
 				case 25:
-					pty[w].attr &= ~A_BLINK;
+					pty[w].cur.attr &= ~A_BLINK;
 					break;
 				case 27:
-					pty[w].attr &= ~A_REVERSE;
+					pty[w].cur.attr &= ~A_REVERSE;
 					break;
 				case 28:
-					pty[w].attr &= ~A_INVISIBLE;
+					pty[w].cur.attr &= ~A_INVISIBLE;
 					break;
 				case 100:
-					pty[w].fg = pty[w].bg = (short)-1;
+					pty[w].cur.fg =
+					pty[w].cur.bg = (short)-1;
 					break;
 				default:
 					n = pty[w].escparam[i];
 					if (n >= 30 && n <= 37)
-						pty[w].fg = n % 10;
+						pty[w].cur.fg = n % 10;
 					else if (n >= 40 && n <= 47)
-						pty[w].bg = n % 10;
+						pty[w].cur.bg = n % 10;
 					break;
 			}
 			tflush();
 			break;
 		case 's':
 			if (pty[w].escmode != ']') break;
-			pty[w].save_x = pty[w].cur_x;
-			pty[w].save_y = pty[w].cur_y;
+			memcpy((char *)&(pty[w].save), (char *)&(pty[w].cur),
+				sizeof(pty[w].save));
 			break;
 		case 'u':
 			if (pty[w].escmode != ']') break;
-			pty[w].cur_x = pty[w].save_x;
-			pty[w].cur_y = pty[w].save_y;
-			biasxy(w, &(pty[w].cur_x), &(pty[w].cur_y));
+			memcpy((char *)&(pty[w].cur), (char *)&(pty[w].save),
+				sizeof(pty[w].cur));
+			biasxy(w, &(pty[w].cur.x), &(pty[w].cur.y));
 			surelocate(w, 1);
 			tflush();
 			break;
@@ -991,12 +994,12 @@ int w, c, fd;
 			switch (pty[w].escparam[0]) {
 				case -1:
 				case 0:
-					min = pty[w].cur_y;
+					min = pty[w].cur.y;
 					max = pty[w].max_y;
 					break;
 				case 1:
 					min = pty[w].min_y;
-					max = pty[w].cur_y + 1;
+					max = pty[w].cur.y + 1;
 					break;
 				case 2:
 					min = pty[w].min_y;
@@ -1029,7 +1032,7 @@ int w, c, fd;
 					putterm(L_CLEARBOL);
 					break;
 				case 2:
-					locate(pty[w].min_x, pty[w].cur_y);
+					locate(pty[w].min_x, pty[w].cur.y);
 					putterm(L_CLEAR);
 					surelocate(w, 1);
 					break;
@@ -1109,10 +1112,10 @@ int w, c, fd;
 					if (pty[w].ntabstop <= (u_char)0)
 						break;
 					for (i = 0; i < pty[w].ntabstop; i++)
-						if (pty[w].cur_x
+						if (pty[w].cur.x
 						<= pty[w].tabstop[i])
 							break;
-					if (pty[w].cur_x != pty[w].tabstop[i])
+					if (pty[w].cur.x != pty[w].tabstop[i])
 						break;
 					memmove((char *)&(pty[w].tabstop[i]),
 						(char *)
@@ -1132,15 +1135,15 @@ int w, c, fd;
 			i = 0;
 			switch (pty[w].escparam[0]) {
 				case 5:
-					i = snprintf2(buf, sizeof(buf),
+					i = Xsnprintf(buf, sizeof(buf),
 						"\033[0n");
 					break;
 				case 6:
-					i = snprintf2(buf, sizeof(buf),
+					i = Xsnprintf(buf, sizeof(buf),
 						SIZEFMT,
-						pty[w].cur_y
+						pty[w].cur.y
 						- pty[w].min_y + 1,
-						pty[w].cur_x
+						pty[w].cur.x
 						- pty[w].min_x + 1);
 					break;
 				case 99:
@@ -1221,7 +1224,7 @@ int w;
 
 	switch (pty[w].escmode) {
 		case '\033':
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 			VOID_C evalnf(w, NULL);
 #endif
 			evalescape(w, uc);
@@ -1248,7 +1251,7 @@ int w;
 		default:
 			if (uc >= ' ' && uc != '\177') evalnormal(w, uc);
 			else {
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 				VOID_C evalnf(w, NULL);
 #endif
 				evalcontrol(w, uc);
@@ -1266,32 +1269,33 @@ int n, c;
 			putterm(L_CLEAR);
 			break;
 		case T_NORMAL:
-			pty[MAXWINDOWS].attr = (u_short)0;
-			pty[MAXWINDOWS].fg = pty[MAXWINDOWS].bg = (short)-1;
+			pty[MAXWINDOWS].cur.attr = (u_short)0;
+			pty[MAXWINDOWS].cur.fg =
+			pty[MAXWINDOWS].cur.bg = (short)-1;
 			break;
 		case T_BOLD:
-			pty[MAXWINDOWS].attr |= A_BOLD;
+			pty[MAXWINDOWS].cur.attr |= A_BOLD;
 			break;
 		case T_REVERSE:
-			pty[MAXWINDOWS].attr |= A_REVERSE;
+			pty[MAXWINDOWS].cur.attr |= A_REVERSE;
 			break;
 		case T_DIM:
-			pty[MAXWINDOWS].attr |= A_DIM;
+			pty[MAXWINDOWS].cur.attr |= A_DIM;
 			break;
 		case T_BLINK:
-			pty[MAXWINDOWS].attr |= A_BLINK;
+			pty[MAXWINDOWS].cur.attr |= A_BLINK;
 			break;
 		case T_STANDOUT:
-			pty[MAXWINDOWS].attr |= A_STANDOUT;
+			pty[MAXWINDOWS].cur.attr |= A_STANDOUT;
 			break;
 		case T_UNDERLINE:
-			pty[MAXWINDOWS].attr |= A_UNDERLINE;
+			pty[MAXWINDOWS].cur.attr |= A_UNDERLINE;
 			break;
 		case END_STANDOUT:
-			pty[MAXWINDOWS].attr &= ~A_STANDOUT;
+			pty[MAXWINDOWS].cur.attr &= ~A_STANDOUT;
 			break;
 		case END_UNDERLINE:
-			pty[MAXWINDOWS].attr &= ~A_UNDERLINE;
+			pty[MAXWINDOWS].cur.attr &= ~A_UNDERLINE;
 			break;
 		case L_INSERT:
 		case L_DELETE:
@@ -1306,13 +1310,13 @@ int n, c;
 			tflush();
 			break;
 		case C_HOME:
-			pty[MAXWINDOWS].cur_x =
-			pty[MAXWINDOWS].cur_y = (short)0;
+			pty[MAXWINDOWS].cur.x =
+			pty[MAXWINDOWS].cur.y = (short)0;
 			reallocate(MAXWINDOWS, 0, 0);
 			break;
 		case C_RETURN:
 			reallocate(MAXWINDOWS,
-				pty[MAXWINDOWS].min_x, pty[MAXWINDOWS].cur_y);
+				pty[MAXWINDOWS].min_x, pty[MAXWINDOWS].cur.y);
 			break;
 		case C_NEWLINE:
 			while (c--) evallf(MAXWINDOWS);
@@ -1320,26 +1324,26 @@ int n, c;
 		case C_UP:
 		case C_NUP:
 			reallocate(MAXWINDOWS,
-				pty[MAXWINDOWS].cur_x,
-				pty[MAXWINDOWS].cur_y - c);
+				pty[MAXWINDOWS].cur.x,
+				pty[MAXWINDOWS].cur.y - c);
 			break;
 		case C_DOWN:
 		case C_NDOWN:
 			reallocate(MAXWINDOWS,
-				pty[MAXWINDOWS].cur_x,
-				pty[MAXWINDOWS].cur_y + c);
+				pty[MAXWINDOWS].cur.x,
+				pty[MAXWINDOWS].cur.y + c);
 			break;
 		case C_RIGHT:
 		case C_NRIGHT:
 			reallocate(MAXWINDOWS,
-				pty[MAXWINDOWS].cur_x + c,
-				pty[MAXWINDOWS].cur_y);
+				pty[MAXWINDOWS].cur.x + c,
+				pty[MAXWINDOWS].cur.y);
 			break;
 		case C_LEFT:
 		case C_NLEFT:
 			reallocate(MAXWINDOWS,
-				pty[MAXWINDOWS].cur_x - c,
-				pty[MAXWINDOWS].cur_y);
+				pty[MAXWINDOWS].cur.x - c,
+				pty[MAXWINDOWS].cur.y);
 			break;
 		default:
 			return(n);
@@ -1362,28 +1366,28 @@ int n;
 	if (ptyrecvbuf(&w1, sizeof(w1)) < 0) return(0);
 
 	switch (n) {
-		case TE_PUTCH2:
+		case TE_XPUTCH:
 			surelocate(MAXWINDOWS, 0);
 			settermattr(MAXWINDOWS);
 			settermcode(MAXWINDOWS);
-			i = cprintf2("%c", w1);
-			pty[MAXWINDOWS].cur_x += i;
+			i = Xcprintf("%c", w1);
+			pty[MAXWINDOWS].cur.x += i;
 			last_x += i;
 			tflush();
 			break;
-		case TE_CPUTS2:
-			s = malloc2(w1 + 1);
+		case TE_XCPUTS:
+			s = Xmalloc(w1 + 1);
 			if (ptyrecvbuf(s, w1) >= 0) {
 				surelocate(MAXWINDOWS, 0);
 				settermattr(MAXWINDOWS);
 				settermcode(MAXWINDOWS);
 				s[w1] = '\0';
-				i = cprintf2("%s", s);
-				pty[MAXWINDOWS].cur_x += i;
+				i = Xcprintf("%s", s);
+				pty[MAXWINDOWS].cur.x += i;
 				last_x += i;
 				tflush();
 			}
-			free(s);
+			Xfree(s);
 			break;
 		case TE_PUTTERM:
 			if ((n = chgattr(w1, 1)) < 0) break;
@@ -1414,25 +1418,25 @@ int n;
 			reallocate(MAXWINDOWS, w1, w2);
 			for (i = 0; i < MAXWINDOWS; i++) {
 				if (ptylist[i].pid) continue;
-				pty[i].cur_x = w1;
-				pty[i].cur_y = w2;
-				biasxy(i, &(pty[i].cur_x), &(pty[i].cur_y));
+				pty[i].cur.x = w1;
+				pty[i].cur.y = w2;
+				biasxy(i, &(pty[i].cur.x), &(pty[i].cur.y));
 			}
 			break;
 		case TE_CPUTNL:
-			pty[MAXWINDOWS].cur_x = (short)0;
+			pty[MAXWINDOWS].cur.x = (short)0;
 			evallf(MAXWINDOWS);
 			break;
 		case TE_CHGCOLOR:
 			if (ptyrecvword(&n) < 0) break;
 
 			if (n) {
-				pty[MAXWINDOWS].fg = (w1 == ANSI_BLACK)
+				pty[MAXWINDOWS].cur.fg = (w1 == ANSI_BLACK)
 					? ANSI_WHITE : ANSI_BLACK;
-				pty[MAXWINDOWS].bg = w1;
+				pty[MAXWINDOWS].cur.bg = w1;
 			}
 			else {
-				pty[MAXWINDOWS].fg = w1;
+				pty[MAXWINDOWS].cur.fg = w1;
 			}
 			break;
 		case TE_MOVECURSOR:
@@ -1451,12 +1455,12 @@ int n;
 			win = w1;
 			if (pid < (p_id_t)0) break;
 			resetptyterm(win, (ptylist[win].pid) ? 0 : 1);
-			biasxy(win, &(pty[win].cur_x), &(pty[win].cur_y));
+			biasxy(win, &(pty[win].cur.x), &(pty[win].cur.y));
 			ptylist[win].pid = pid;
 			surelocate(win, 1);
 			tflush();
 			if (pid) {
-				VOID_C setwsize(ptylist[win].fd,
+				setwsize(ptylist[win].fd,
 					pty[win].max_x - pty[win].min_x,
 					pty[win].max_y - pty[win].min_y);
 				break;
@@ -1480,7 +1484,7 @@ int n;
 			for (i = 0; i < n; i++) {
 				winvar[i].v_fileperrow = row[i];
 				resetptyterm(i, 0);
-				VOID_C setwsize(ptylist[i].fd,
+				setwsize(ptylist[i].fd,
 					pty[i].max_x - pty[i].min_x,
 					pty[i].max_y - pty[i].min_y);
 			}
@@ -1522,7 +1526,7 @@ int n;
 			pty[w1].termflags &= ~T_LOCKED;
 			sendbuf(ptylist[w1].fd, "\n", 1);
 			break;
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 		case TE_CHANGEKCODE:
 			if (ptyrecvbuf(&w2, sizeof(w2)) < 0) break;
 			inputkcode = w1;
@@ -1536,13 +1540,13 @@ int n;
 			if (ptyrecvbuf(&w2, sizeof(w2)) < 0) break;
 			ptylist[w1].outcode = (u_char)w2;
 			break;
-#endif	/* !_NOKANJICONV */
+#endif	/* DEP_KCONV */
 		case TE_AWAKECHILD:
 			if (ptyrecvbuf(&n, sizeof(n)) < 0
 			|| ptyrecvstring(&s) < 0)
 				break;
 			if (ptyrecvstring(&arg) < 0) {
-				if (s) free(s);
+				Xfree(s);
 				break;
 			}
 			if (ptyrecvstring(&cwd) < 0) cwd = NULL;
@@ -1552,9 +1556,9 @@ int n;
 			sendstring(ptylist[w1].fd, s);
 			sendstring(ptylist[w1].fd, arg);
 			sendstring(ptylist[w1].fd, cwd);
-			if (s) free(s);
-			if (arg) free(arg);
-			if (cwd) free(cwd);
+			Xfree(s);
+			Xfree(arg);
+			Xfree(cwd);
 			break;
 		default:
 			break;
@@ -1565,20 +1569,20 @@ int n;
 
 static int NEAR evalinput(VOID_A)
 {
-#ifdef	_NOKANJICONV
-	char buf[MAXKLEN];
-#else
+#ifdef	DEP_KCONV
 	u_short ubuf[MAXNFLEN];
 	char buf[MAXKANJIBUF + 1], buf2[MAXKANJIBUF + 1];
 	u_int u;
 	int i, ch, len, cnv, incode, outcode;
+#else
+	char buf[MAXKLEN];
 #endif
 	keyseq_t key;
 	int n;
 
 	if (ptyrecvbuf(&(key.code), sizeof((key.code))) < 0) return(0);
 
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	cnv = 0;
 	incode = (inputkcode != NOCNV) ? inputkcode : DEFCODE;
 	outcode = (win < MAXWINDOWS && ptylist[win].incode != NOCNV)
@@ -1591,24 +1595,24 @@ static int NEAR evalinput(VOID_A)
 		buf[0] = K_ESC;
 		buf[1] = (key.code & 0xff);
 	}
-#ifdef	_NOKANJICONV
-# ifdef	CODEEUC
-	else if (isekana2(key.code)) {
-		key.len = (u_char)2;
-		key.str = buf;
-		buf[0] = C_EKANA;
-		buf[1] = (key.code & 0xff);
-	}
-# endif
-#else	/* !_NOKANJICONV */
+#ifdef	DEP_KCONV
 	else if (incode == EUC && isekana2(key.code)) {
 		if (incode != outcode) cnv++;
 		key.len = (u_char)2;
 		key.str = buf;
-		buf[0] = C_EKANA;
+		buf[0] = (char)C_EKANA;
 		buf[1] = (key.code & 0xff);
 	}
-#endif	/* !_NOKANJICONV */
+#else	/* !DEP_KCONV */
+# ifdef	CODEEUC
+	else if (isekana2(key.code)) {
+		key.len = (u_char)2;
+		key.str = buf;
+		buf[0] = (char)C_EKANA;
+		buf[1] = (key.code & 0xff);
+	}
+# endif
+#endif	/* !DEP_KCONV */
 	else if (!(key.code & K_ALTERNATE) && key.code > K_MAX) {
 		n = directoutput(key.code);
 		if (win < MAXWINDOWS) {
@@ -1623,12 +1627,12 @@ static int NEAR evalinput(VOID_A)
 		key.len = (u_char)1;
 		key.str = buf;
 		buf[0] = key.code;
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 		if (incode == outcode) /*EMPTY*/;
-		else if (incode == SJIS && iskana2(key.code)) cnv++;
+		else if (incode == SJIS && Xiskana(key.code)) cnv++;
 		else if (incode == EUC && key.code == C_EKANA
 		&& (n = ptygetch()) >= 0) {
-			if (!iskana2(n)) ptyungetch((u_char)n);
+			if (!Xiskana(n)) ptyungetch((u_char)n);
 			else {
 				cnv++;
 				buf[1] = n;
@@ -1671,10 +1675,10 @@ static int NEAR evalinput(VOID_A)
 			buf[1] = n;
 			(key.len)++;
 		}
-#endif	/* !_NOKANJICONV */
+#endif	/* DEP_KCONV */
 	}
 
-#ifndef	_NOKANJICONV
+#ifdef	DEP_KCONV
 	if (cnv) {
 		buf[key.len] = '\0';
 		key.str = (char *)ptykconv(buf, buf2, incode, outcode);
@@ -1698,7 +1702,7 @@ int backend(VOID_A)
 #ifdef	SIGWINCH
 	ptywinched = 0;
 	pollscreen(-1);
-	signal2(SIGWINCH, (sigcst_t)wintr);
+	VOID_C signal2(SIGWINCH, (sigcst_t)wintr);
 #endif
 
 	ttyiomode(0);
@@ -1712,7 +1716,7 @@ int backend(VOID_A)
 		for (i = 0; i < MAXWINDOWS; i++)
 			fds[i] = (ptylist[i].pid) ? ptylist[i].fd : -1;
 		fds[i] = emufd;
-		if (selectpty(MAXWINDOWS + 1, fds, result, -1) <= 0) continue;
+		if (selectpty(MAXWINDOWS + 1, fds, result, -1L) <= 0) continue;
 
 		if (result[MAXWINDOWS] && (n = evalinput()) > 0) continue;
 
@@ -1735,4 +1739,4 @@ int backend(VOID_A)
 
 	return(0);
 }
-#endif	/* !_NOPTY */
+#endif	/* DEP_PTY */
